/*
 * This source file was generated by the Gradle 'init' task
 */
package be.kuleuven;

import java.util.Scanner;

import javax.persistence.Persistence;

public class App {

    public static void main(String[] args) {
        ConnectionManager cm = new ConnectionManager("jdbc:mysql://localhost:3306/school?allowMultiQueries=true",
                "root", "");
        cm.initTables();

        // // JPA
        var sessionFactory = Persistence.createEntityManagerFactory("be.kuleuven.studenthibernate");
        var entityManager = sessionFactory.createEntityManager();
        StudentRepository sr = new StudentRepositoryJPAimpl(entityManager);

        // Soort DASHBOARD applicatie
        boolean quit = false;
        Scanner scanner = new Scanner(System.in);
        while (!quit) {
            System.out.println("\nWelkom bij de student database");
            System.out.println("Kies een actie:");
            System.out.println("1. Create Student");
            System.out.println("2. Get Student");
            System.out.println("3. Update Student");
            System.out.println("4. Delete Student");
            System.out.println("5. Get all Students");
            System.out.println("6. Quit");
            System.out.print("Voer je keuze in: ");

            int keuze;
            if (scanner.hasNextInt()) {
                keuze = scanner.nextInt();
                switch (keuze) {
                    case 1 -> {
                        System.out.println("\nCREATE\n");
                        create(sr, scanner);
                    }
                    case 2 -> {
                        System.out.println("\nREAD\n");
                        read(sr, scanner);
                    }
                    case 3 -> {
                        System.out.println("\nUPDATE\n");
                        update(sr, scanner);
                    }
                    case 4 -> {
                        System.out.println("\nDELETE\n");
                        delete(sr, scanner);
                    }
                    case 5 -> {
                        System.out.println("\nREAD ALL\n");
                        readall(sr, scanner);
                    }
                    case 6 -> {
                        System.out.println("\nQUITTING\n");
                        quit = true;
                    }
                    default -> {
                        System.out.println("Ongeldige keuze, probeer opnieuw!");
                        return;
                    }
                }
            } else {
                System.out.println("Ongeldige invoer, probeer opnieuw.");
                scanner.next(); // Verwijder ongeldige invoer
            }
        }
        scanner.close();
        cm.flushConnection();
    }

    private static void create(StudentRepository sr, Scanner scanner) {
        System.out.println("\nEnter student details:");

        // Get student number
        System.out.print("Student Nummer: ");
        int studnr = scanner.nextInt();
        scanner.nextLine(); // Consume newline

        // Get name
        System.out.print("Voornaam: ");
        String voornaam = scanner.nextLine();

        // Get last name
        System.out.print("Achternaam: ");
        String naam = scanner.nextLine();

        // Get goedbezig status
        System.out.print("Goed Bezig? (yes/no, default = no): ");
        boolean goedbezig = scanner.nextLine().equalsIgnoreCase("yes");

        // Create student
        Student student = new Student(studnr, naam, voornaam, goedbezig);

        try {
            sr.addStudentToDb(student);
            // Show confirmation
            System.out.println("\nCreated student:");
            System.out.println(student);
        } catch (RuntimeException e) {
            System.out.println("\n### Error creating student. Initiating rollback ...");
        }
    }

    private static void read(StudentRepository sr, Scanner scanner) {
        System.out.println("\nEnter student details:");

        // Get student number
        System.out.print("Student Nummer: ");
        int studnr = scanner.nextInt();
        scanner.nextLine(); // Consume newline

        try {
            Student student = sr.getStudentsByStudnr(studnr);
            // Show confirmation
            System.out.println("\nFound student:");
            System.out.println(student);
        } catch (RuntimeException e) {
            System.out.println("\n### Error getting student.");
        }
    }

    private static void update(StudentRepository sr, Scanner scanner) {
        System.out.println("\nEnter student details:");

        // Get student number
        System.out.print("Student Nummer: ");
        int studnr = scanner.nextInt();
        scanner.nextLine(); // Consume newline

        try {
            Student student = sr.getStudentsByStudnr(studnr);
            // Show confirmation
            System.out.println("\nCurrent student:");
            System.out.println(student);
        } catch (RuntimeException e) {
            System.out.println("\n### Error getting student for updating.");
            return;
        }

        // Get name
        System.out.print("enter New Voornaam: ");
        String voornaam = scanner.nextLine();

        // Get last name
        System.out.print("enter New Achternaam: ");
        String naam = scanner.nextLine();

        // Get goedbezig status
        System.out.print("enter New Goed Bezig? (yes/no, default = no): ");
        boolean goedbezig = scanner.nextLine().equalsIgnoreCase("yes");

        // Create student
        Student student = new Student(studnr, naam, voornaam, goedbezig);

        try {
            sr.updateStudentInDb(student);
            // Show confirmation
            System.out.println("\nUpdated student:");
            System.out.println(student);
        } catch (RuntimeException e) {
            System.out.println("\n### Error updating student. Initiating rollback ...");
        }
    }

    private static void delete(StudentRepository sr, Scanner scanner) {
        System.out.println("\nEnter student details:");

        // Get student number
        System.out.print("Student Nummer: ");
        int studnr = scanner.nextInt();
        scanner.nextLine(); // Consume newline

        try {
            Student student = sr.getStudentsByStudnr(studnr);
            // Show confirmation
            System.out.println("\nCurrent student:");
            System.out.println(student);
        } catch (RuntimeException e) {
            System.out.println("\n### Error getting student for deletion.");
            return;
        }

        try {
            sr.deleteStudentInDb(studnr);
            // Show confirmation
            System.out.println("\nDeleted student with student number:");
            System.out.println(studnr);
        } catch (RuntimeException e) {
            System.out.println("\n### Error deleting student. Initiating rollback ...");
        }
    }

    private static void readall(StudentRepository sr, Scanner scanner) {
        try {
            // Show confirmation
            System.out.println("\nFound students:");
            for (Student s : sr.getAllStudents()) {
                System.out.println(s);
            }
        } catch (RuntimeException e) {
            System.out.println("\n### Error getting students.");
        }
    }
}
